@@ -20,6 +20,8 @@
 #include <interfaces/icore.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/iruncontroller.h>
+#include <interfaces/idocumentcontroller.h>
+#include <interfaces/iprojectcontroller.h>
 #include <language/backgroundparser/backgroundparser.h>
 #include <interfaces/iproject.h>
 #include <klocalizedstring.h>
@@ -42,16 +44,27 @@ ParseProjectJob::~ParseProjectJob() {
 }
 
 ParseProjectJob::ParseProjectJob(IProject* project, bool forceUpdate)
+    : m_updated(0)
+    , m_forceUpdate(forceUpdate)
+    , m_project(project)
 {
     connect(project, SIGNAL(destroyed(QObject*)), SLOT(deleteNow()));
-    m_project = project;
-    m_updated = 0;
-    m_forceUpdate = forceUpdate;
-    m_totalFiles = project->fileSet().size();
+
+    if (!ICore::self()->projectController()->parseAllProjectSources()) {
+        // In case we don't want to parse the whole project, still add all currently open files that belong to the project to the background-parser
+        for (auto document: ICore::self()->documentController()->openDocuments()) {
+            const auto path = IndexedString(document->url());
+            if (project->fileSet().contains(path)) {
+                m_filesToParse.insert(path);
+            }
+        }
+    } else {
+        m_filesToParse = project->fileSet();
+    }
 
     setCapabilities(Killable);
 
-    setObjectName(i18np("Process 1 file in %2","Process %1 files in %2", m_totalFiles, m_project->name()));
+    setObjectName(i18np("Process 1 file in %2","Process %1 files in %2", m_filesToParse.size(), m_project->name()));
 }
 
 void ParseProjectJob::deleteNow() {
@@ -66,10 +79,10 @@ void ParseProjectJob::updateReady(const IndexedString& url, ReferencedTopDUConte
     Q_UNUSED(url);
     Q_UNUSED(topContext);
     ++m_updated;
-    if(m_updated % ((m_totalFiles / 100)+1) == 0)
+    if(m_updated % ((m_filesToParse.size() / 100)+1) == 0)
         updateProgress();
 
-    if(m_updated >= m_totalFiles)
+    if(m_updated >= m_filesToParse.size())
         deleteLater();
 }
 
@@ -79,9 +92,8 @@ void ParseProjectJob::start() {
     }
 
     kDebug() << "starting project parse job";
-    QSet< IndexedString > files = m_project->fileSet();
 
-    TopDUContext::Features processingLevel = files.size() < ICore::self()->languageController()->completionSettings()->minFilesForSimplifiedParsing() ?
+    TopDUContext::Features processingLevel = m_filesToParse.size() < ICore::self()->languageController()->completionSettings()->minFilesForSimplifiedParsing() ?
                                     TopDUContext::VisibleDeclarationsAndContexts : TopDUContext::SimplifiedVisibleDeclarationsAndContexts;
 
     if (m_forceUpdate) {
@@ -90,11 +102,33 @@ void ParseProjectJob::start() {
         }
         processingLevel = (TopDUContext::Features)(TopDUContext::ForceUpdate | processingLevel);
     }
+
+    if (auto currentDocument = ICore::self()->documentController()->activeDocument()) {
+        const auto path = IndexedString(currentDocument->url());
+        if (m_filesToParse.contains(path)) {
+            ICore::self()->languageController()->backgroundParser()->addDocument(path, TopDUContext::AllDeclarationsContextsAndUses, BackgroundParser::BestPriority, this);
+            m_filesToParse.remove(path);
+        }
+    }
+
+    // Add all currently open files that belong to the project to the background-parser, so that they'll be parsed first of all
+    for (auto document: ICore::self()->documentController()->openDocuments()) {
+        const auto path = IndexedString(document->url());
+        if (m_filesToParse.contains(path)) {
+            ICore::self()->languageController()->backgroundParser()->addDocument(path, TopDUContext::AllDeclarationsContextsAndUses, 10, this );
+            m_filesToParse.remove(path);
+        }
+    }
+
+    if (!ICore::self()->projectController()->parseAllProjectSources()) {
+        return;
+    }
+
     // prevent UI-lockup by processing events after some files
     // esp. noticeable when dealing with huge projects
     const int processAfter = 1000;
     int processed = 0;
-    foreach(const IndexedString& url, files) {
+    foreach(const IndexedString& url, m_filesToParse) {
         ICore::self()->languageController()->backgroundParser()->addDocument( url, processingLevel, BackgroundParser::InitialParsePriority, this );
         ++processed;
         if (processed == processAfter) {
