@@ -58,7 +58,7 @@ public:
   bool m_inSymbolTable : 1;
   bool m_isTypeAlias   : 1;
   bool m_anonymousInContext : 1; //Whether the declaration was added into the parent-context anonymously
-  bool m_isFinal       : 1;
+  bool m_isDeprecated      : 1;
   bool m_alwaysForceDirect : 1;
   bool m_isAutoDeclaration : 1;
   bool m_isExplicitlyDeleted : 1;
