@@ -299,7 +299,7 @@ class DUChainPrivate
       DUChainPrivate* m_data;
   };
 public:
-  DUChainPrivate() : m_chainsMutex(QMutex::Recursive), instance(0), m_cleanupDisabled(false), m_destroyed(false), m_environmentListInfo("Environment Lists"), m_environmentInfo("Environment Information")
+  DUChainPrivate() : m_chainsMutex(QMutex::Recursive), m_cleanupMutex(QMutex::Recursive), instance(0), m_cleanupDisabled(false), m_destroyed(false), m_environmentListInfo("Environment Lists"), m_environmentInfo("Environment Information")
   {
 #if defined(TEST_NO_CLEANUP)
     m_cleanupDisabled = true;
@@ -419,6 +419,8 @@ public:
   ///Should be released during expensive disk-operations and such.
   QMutex m_chainsMutex;
 
+  QMutex m_cleanupMutex;
+
   CleanupThread* m_cleanup;
 
   DUChain* instance;
@@ -675,10 +677,9 @@ public:
       }
     }
   }
-  
+
   QMutex& cleanupMutex() {
-    static QMutex mutex(QMutex::Recursive);
-    return mutex;
+    return m_cleanupMutex;
   }
 
   ///@param retries When this is nonzero, then doMoreCleanup will do the specified amount of cycles
@@ -1572,6 +1573,10 @@ void DUChain::shutdown()
 
   sdDUChainPrivate->doMoreCleanup(); //Must be done _before_ finalCleanup, else we may be deleting yet needed data
 
+  sdDUChainPrivate->m_openDocumentContexts.clear();
+  sdDUChainPrivate->m_destroyed = true;
+  sdDUChainPrivate->clear();
+
   {
     //Acquire write-lock of the repository, so when kdevelop crashes in that process, the repository is discarded
     //Crashes here may happen in an inconsistent state, thus this makes sense, to protect the user from more crashes
@@ -1579,12 +1584,7 @@ void DUChain::shutdown()
     finalCleanup();
     globalItemRepositoryRegistry().unlockForWriting();
   }
-  
-  sdDUChainPrivate->doMoreCleanup();
-  sdDUChainPrivate->m_openDocumentContexts.clear();
-  sdDUChainPrivate->m_destroyed = true;
-  sdDUChainPrivate->clear();
-  
+
   globalItemRepositoryRegistry().shutdown();
 }
 
