@@ -52,12 +52,17 @@ class DUChainItemFactory : public DUChainBaseFactory {
   void copy(const DUChainBaseData& from, DUChainBaseData& to, bool constant) const {
     Q_ASSERT(from.classId == T::Identity);
 
-    bool previousConstant = DUChainBaseData::shouldCreateConstantData();
-    DUChainBaseData::setShouldCreateConstantData(constant);
+    bool& isConstant = DUChainBaseData::shouldCreateConstantData();
+    const bool previousConstant = isConstant;
+    if (previousConstant != constant) {
+      isConstant = constant;
+    }
     
     new (&to) Data(static_cast<const Data&>(from)); //Call the copy constructor to initialize the target
     
-    DUChainBaseData::setShouldCreateConstantData(previousConstant);
+    if (previousConstant != constant) {
+      isConstant = previousConstant;
+    }
   }
   
   void callDestructor(DUChainBaseData* data) const {
