@@ -54,6 +54,16 @@ ICodeHighlighting* ILanguageSupport::codeHighlighting() const {
     return 0;
 }
 
+BasicRefactoring* ILanguageSupport::refactoring() const
+{
+    return nullptr;
+}
+
+ICreateClassHelper* ILanguageSupport::createClassHelper() const {
+    return 0;
+}
+
+
 ILanguage* ILanguageSupport::language() {
     return ICore::self()->languageController()->language(name());
 }
@@ -63,6 +73,16 @@ DocumentChangeTracker* ILanguageSupport::createChangeTrackerForDocument ( KTextE
     return new DocumentChangeTracker(document);
 }
 
+ILanguageSupport::WhitespaceSensitivity ILanguageSupport::whitespaceSensititivy() const
+{
+    return ILanguageSupport::Insensitive;
+}
+
+SourceFormatterItemList ILanguageSupport::sourceFormatterItems() const
+{
+    return SourceFormatterItemList();
+}
+
 QString ILanguageSupport::indentationSample() const
 {
     return "";
