@@ -57,7 +57,7 @@ KDevDocumentView::KDevDocumentView( KDevDocumentViewPlugin *plugin, QWidget *par
 
     m_documentModel = new KDevDocumentModel(this);
 
-    m_delegate = new KDevDocumentViewDelegate( this, this );
+    m_delegate = new KDevDocumentViewDelegate( this );
 
     m_proxy = new QSortFilterProxyModel( this );
     m_proxy->setSourceModel( m_documentModel );
@@ -78,7 +78,6 @@ KDevDocumentView::KDevDocumentView( KDevDocumentViewPlugin *plugin, QWidget *par
 
     setFocusPolicy( Qt::NoFocus );
 
-    setRootIsDecorated( false );
     header()->hide();
 
     setSelectionBehavior( QAbstractItemView::SelectRows );
@@ -98,24 +97,22 @@ void KDevDocumentView::mousePressEvent( QMouseEvent * event )
     QModelIndex proxyIndex = indexAt( event->pos() );
     QModelIndex index = m_proxy->mapToSource( proxyIndex );
 
-    if ( event->button() == Qt::LeftButton && proxyIndex.parent().isValid() &&
-            event->modifiers() == Qt::NoModifier )
-    {
-        KDevelop::IDocumentController* dc = m_plugin->core()->documentController();
-        KUrl documentUrl = static_cast<KDevDocumentItem*>( m_documentModel->itemFromIndex( index ) )->fileItem()->url();
-        if (dc->documentForUrl(documentUrl) != dc->activeDocument())
-        {
-            dc->openDocument(documentUrl);
+    if (event->button() == Qt::LeftButton && event->modifiers() == Qt::NoModifier) {
+        if (proxyIndex.parent().isValid()) {
+            // this is a document item
+            KDevelop::IDocumentController* dc = m_plugin->core()->documentController();
+            KUrl documentUrl = static_cast<KDevDocumentItem*>(m_documentModel->itemFromIndex(index))->fileItem()->url();
+            if (dc->documentForUrl(documentUrl) != dc->activeDocument()) {
+                dc->openDocument(documentUrl);
+                return;
+            }
+        } else {
+            // this is a folder item
+            setExpanded(proxyIndex, !isExpanded(proxyIndex));
             return;
         }
     }
 
-    if ( !proxyIndex.parent().isValid() )
-    {
-        setExpanded( proxyIndex, !isExpanded( proxyIndex ) );
-        return;
-    }
-
     QTreeView::mousePressEvent( event );
 }
 
@@ -168,8 +165,16 @@ void KDevDocumentView::reloadSelected()
 
 void KDevDocumentView::contextMenuEvent( QContextMenuEvent * event )
 {
+    QModelIndex proxyIndex = indexAt( event->pos() );
+    QModelIndex index = m_proxy->mapToSource( proxyIndex );
+
+    // for now, ignore clicks on empty space or folder items
+    if (!proxyIndex.isValid() || !proxyIndex.parent().isValid()) {
+        return;
+    }
+
     updateSelectedDocs();
-    
+
     if (!m_selectedDocs.isEmpty())
     {
         KMenu* ctxMenu = new KMenu(this);
@@ -308,35 +313,16 @@ void KDevDocumentView::closed( KDevelop::IDocument* document )
 
 void KDevDocumentView::updateCategoryItem( KDevCategoryItem *item )
 {
-    QString label = QFileInfo(item->url().pathOrUrl()).path();
-
-    foreach ( const KDevelop::IProject* prj, m_projects ) {
-        const QString possibleLabel = prj->relativeUrl( KUrl(label) ).pathOrUrl();
-        if ( !possibleLabel.startsWith( "../" ) )
-            label = possibleLabel;
-        else
-            label.replace( QDir::homePath(), "~" );
+    QString text = KDevelop::ICore::self()->projectController()->prettyFilePath(item->url(), KDevelop::IProjectController::FormatPlain);
+    // remove trailing slash
+    if (text.length() > 1) {
+        text.chop(1);
     }
-
-    item->setText( label );
-}
-
-bool projectPathlongerThan( const KDevelop::IProject* prj1, const KDevelop::IProject* prj2 )
-{
-    // compare path depth of two project folders
-    const int c1 = prj1->folder().pathOrUrl().split( QDir::separator() ).count();
-    const int c2 = prj2->folder().pathOrUrl().split( QDir::separator() ).count();
-    return c1 > c2;
+    item->setText(text);
 }
 
 void KDevDocumentView::updateProjectPaths()
 {
-
-    m_projects = KDevelop::ICore::self()->projectController()->projects();
-
-    // sort folders, longest first, so replacing them one by one is save
-    qSort( m_projects.begin(), m_projects.end(), projectPathlongerThan );
-
     foreach ( KDevCategoryItem *it, m_documentModel->categoryList() )
         updateCategoryItem( it );
 }
