@@ -41,12 +41,27 @@
 #include <KFileDialog>
 #include <KShell>
 #include <interfaces/iplugincontroller.h>
+#include <interfaces/idocumentcontroller.h>
 
 #include "executescriptplugin.h"
 #include <util/kdevstringhandler.h>
 #include <util/environmentgrouplist.h>
 #include <project/projectitemlineedit.h>
 
+static const QString interpreterForUrl(const KUrl& url) {
+    auto mimetype = KMimeType::findByUrl(url);
+    static QHash<QString, QString> knownMimetypes;
+    if ( knownMimetypes.isEmpty() ) {
+        knownMimetypes["text/x-python"] = "python";
+        knownMimetypes["application/x-php"] = "php";
+        knownMimetypes["application/x-ruby"] = "ruby";
+        knownMimetypes["application/x-shellscript"] = "bash";
+        knownMimetypes["application/x-perl"] = "perl -e";
+    }
+    const QString& interp = knownMimetypes.value(mimetype->name());
+    return interp;
+}
+
 KIcon ScriptAppConfigPage::icon() const
 {
     return KIcon("system-run");
@@ -60,14 +75,22 @@ void ScriptAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg, KDevelo
         executablePath->setStartDir( project->folder() );
     }
     
-    interpreter->lineEdit()->setText( cfg.readEntry( ExecuteScriptPlugin::interpreterEntry, "" ) );
+    auto doc = KDevelop::ICore::self()->documentController()->activeDocument();
+    interpreter->lineEdit()->setText( cfg.readEntry( ExecuteScriptPlugin::interpreterEntry,
+                                                     doc ? interpreterForUrl(doc->url()) : "" ) );
     executablePath->setUrl( cfg.readEntry( ExecuteScriptPlugin::executableEntry, "" ) );
     remoteHostCheckbox->setChecked( cfg.readEntry( ExecuteScriptPlugin::executeOnRemoteHostEntry, false ) );
     remoteHost->setText( cfg.readEntry( ExecuteScriptPlugin::remoteHostEntry, "" ) );
+    bool runCurrent = cfg.readEntry( ExecuteScriptPlugin::runCurrentFileEntry, true );
+    if ( runCurrent ) {
+        runCurrentFile->setChecked( true );
+    } else {
+        runFixedFile->setChecked( true );
+    }
     arguments->setText( cfg.readEntry( ExecuteScriptPlugin::argumentsEntry, "" ) );
     workingDirectory->setUrl( cfg.readEntry( ExecuteScriptPlugin::workingDirEntry, KUrl() ) );
-    environment->setCurrentProfile( cfg.readEntry( ExecuteScriptPlugin::environmentGroupEntry, "default" ) );
-    outputFilteringMode->setCurrentIndex( cfg.readEntry( ExecuteScriptPlugin::outputFilteringEntry, 0u ));
+    environment->setCurrentProfile( cfg.readEntry( ExecuteScriptPlugin::environmentGroupEntry, QString() ) );
+    outputFilteringMode->setCurrentIndex( cfg.readEntry( ExecuteScriptPlugin::outputFilteringEntry, 2u ));
     //runInTerminal->setChecked( cfg.readEntry( ExecuteScriptPlugin::useTerminalEntry, false ) );
     blockSignals( b );
 }
@@ -81,10 +104,6 @@ ScriptAppConfigPage::ScriptAppConfigPage( QWidget* parent )
     //Set workingdirectory widget to ask for directories rather than files
     workingDirectory->setMode(KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
 
-    KDevelop::EnvironmentGroupList env( KGlobal::config() );
-    environment->addItems( env.groups() );
-
-
     //connect signals to changed signal
     connect( interpreter->lineEdit(), SIGNAL(textEdited(QString)), SIGNAL(changed()) );
     connect( executablePath->lineEdit(), SIGNAL(textEdited(QString)), SIGNAL(changed()) );
@@ -92,8 +111,7 @@ ScriptAppConfigPage::ScriptAppConfigPage( QWidget* parent )
     connect( arguments, SIGNAL(textEdited(QString)), SIGNAL(changed()) );
     connect( workingDirectory, SIGNAL(urlSelected(KUrl)), SIGNAL(changed()) );
     connect( workingDirectory->lineEdit(), SIGNAL(textEdited(QString)), SIGNAL(changed()) );
-    connect( environment, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
-    connect( remoteHostCheckbox, SIGNAL(toggled(bool)), remoteHost, SLOT(setEnabled(bool)));
+    connect( environment, SIGNAL(currentProfileChanged(QString)), SIGNAL(changed()) );
     //connect( runInTerminal, SIGNAL(toggled(bool)), SIGNAL(changed()) );
 }
 
@@ -104,6 +122,7 @@ void ScriptAppConfigPage::saveToConfiguration( KConfigGroup cfg, KDevelop::IProj
     cfg.writeEntry( ExecuteScriptPlugin::executableEntry, executablePath->url() );
     cfg.writeEntry( ExecuteScriptPlugin::executeOnRemoteHostEntry, remoteHostCheckbox->isChecked() );
     cfg.writeEntry( ExecuteScriptPlugin::remoteHostEntry, remoteHost->text() );
+    cfg.writeEntry( ExecuteScriptPlugin::runCurrentFileEntry, runCurrentFile->isChecked() );
     cfg.writeEntry( ExecuteScriptPlugin::argumentsEntry, arguments->text() );
     cfg.writeEntry( ExecuteScriptPlugin::workingDirEntry, workingDirectory->url() );
     cfg.writeEntry( ExecuteScriptPlugin::environmentGroupEntry, environment->currentProfile() );
@@ -202,18 +221,23 @@ KIcon ScriptAppConfigType::icon() const
     return KIcon("preferences-plugin-script");
 }
 
-bool ScriptAppConfigType::canLaunch(const KUrl& /*file*/) const
+bool ScriptAppConfigType::canLaunch(const KUrl& file) const
 {
-    return false;
+    return ! interpreterForUrl(file).isEmpty();
 }
 
-bool ScriptAppConfigType::canLaunch(KDevelop::ProjectBaseItem* /*item*/) const
+bool ScriptAppConfigType::canLaunch(KDevelop::ProjectBaseItem* item) const
 {
-    return false;
+    return ! interpreterForUrl(item->path().toUrl()).isEmpty();
 }
 
-void ScriptAppConfigType::configureLaunchFromItem(KConfigGroup /*config*/, KDevelop::ProjectBaseItem* /*item*/) const
+void ScriptAppConfigType::configureLaunchFromItem(KConfigGroup config, KDevelop::ProjectBaseItem* item) const
 {
+    config.writeEntry(ExecuteScriptPlugin::executableEntry, item->path().toUrl());
+    config.writeEntry(ExecuteScriptPlugin::interpreterEntry, interpreterForUrl(item->path().toUrl()));
+    config.writeEntry(ExecuteScriptPlugin::outputFilteringEntry, 2u);
+    config.writeEntry(ExecuteScriptPlugin::runCurrentFileEntry, false);
+    config.sync();
 }
 
 void ScriptAppConfigType::configureLaunchFromCmdLineArguments(KConfigGroup cfg, const QStringList &args) const
@@ -222,6 +246,7 @@ void ScriptAppConfigType::configureLaunchFromCmdLineArguments(KConfigGroup cfg,
     cfg.writeEntry( ExecuteScriptPlugin::interpreterEntry, a.takeFirst() );
     cfg.writeEntry( ExecuteScriptPlugin::executableEntry, a.takeFirst() );
     cfg.writeEntry( ExecuteScriptPlugin::argumentsEntry, KShell::joinArgs(a) );
+    cfg.writeEntry( ExecuteScriptPlugin::runCurrentFileEntry, false );
     cfg.sync();
 }
 
