@@ -32,6 +32,7 @@
 #include <QDateTime>
 #include <QFileSystemWatcher>
 #include <QTimer>
+#include <QTextCodec>
 
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
@@ -54,8 +55,10 @@
 #include <KStandardDirs>
 #include <KTextEdit>
 #include <KDirWatch>
+#include <KTextEditor/Document>
 #include "gitjob.h"
 #include "gitmessagehighlighter.h"
+#include "gitplugincheckinrepositoryjob.h"
 
 K_PLUGIN_FACTORY(KDevGitFactory, registerPlugin<GitPlugin>(); )
 K_EXPORT_PLUGIN(KDevGitFactory(KAboutData("kdevgit","kdevgit",ki18n("Git"),"0.1",ki18n("A plugin to support git version control systems"), KAboutData::License_GPL)))
@@ -219,6 +222,11 @@ bool GitPlugin::hasModifications(const QDir& d)
     return !emptyOutput(lsFiles(d, QStringList("-m"), OutputJob::Silent));
 }
 
+bool GitPlugin::hasModifications(const QDir& repo, const KUrl& file)
+{
+    return !emptyOutput(lsFiles(repo, QStringList() << "-m" << file.path(), OutputJob::Silent));
+}
+
 void GitPlugin::additionalMenuEntries(QMenu* menu, const KUrl::List& urls)
 {
     m_urls = urls;
@@ -331,6 +339,7 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
         connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitStatusOutput_old(KDevelop::DVcsJob*)));
     } else {
         *job << "git" << "status" << "--porcelain";
+        job->setIgnoreError(true);
         connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitStatusOutput(KDevelop::DVcsJob*)));
     }
     *job << "--" << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
@@ -368,6 +377,21 @@ VcsJob* GitPlugin::revert(const KUrl::List& localLocations, IBasicVersionControl
     if(localLocations.isEmpty() )
         return errorsFound(i18n("Could not revert changes"), OutputJob::Verbose);
     
+    QDir repo = urlDir(repositoryRoot(localLocations.first()));
+    QString modified;
+    for (const auto& file: localLocations) {
+        if (hasModifications(repo, file)) {
+            modified.append(file.pathOrUrl() + "<br/>");
+        }
+    }
+    if (!modified.isEmpty()) {
+        auto res = KMessageBox::questionYesNo(nullptr, i18n("The following files have uncommited changes, "
+                                              "which will be lost. Continue?") + "<br/><br/>" + modified);
+        if (res != KMessageBox::Yes) {
+            return errorsFound(QString(), OutputJob::Silent);
+        }
+    }
+
     DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
     job->setType(VcsJob::Revert);
     *job << "git" << "checkout" << "--";
@@ -654,7 +678,8 @@ VcsJob* GitPlugin::renameBranch(const KUrl& repository, const QString& oldBranch
 VcsJob* GitPlugin::currentBranch(const KUrl& repository)
 {
     DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
-    *job << "git" << "rev-parse" << "--abbrev-ref" << "HEAD";
+    job->setIgnoreError(true);
+    *job << "git" << "symbolic-ref" << "-q" << "--short" << "HEAD";
     connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
     return job;
 }
@@ -663,11 +688,6 @@ void GitPlugin::parseGitCurrentBranch(DVcsJob* job)
 {
     QString out = job->output().trimmed();
 
-    // in detached state, we'd like to return an empty string
-    if (out == "HEAD") {
-        out.clear();
-    }
-
     job->setResults(out);
 }
 
@@ -1269,6 +1289,16 @@ VcsJob* GitPlugin::copy(const KUrl& localLocationSrc, const KUrl& localLocationD
 VcsJob* GitPlugin::move(const KUrl& source, const KUrl& destination)
 {
     QDir dir = urlDir(source);
+
+    QFileInfo fileInfo(source.toLocalFile());
+    if (fileInfo.isDir()) {
+        if (isEmptyDirStructure(QDir(source.toLocalFile()))) {
+            //move empty folder, git doesn't do that
+            kDebug() << "empty folder" << source;
+            return new StandardJob(this, KIO::move(source, destination), KDevelop::OutputJob::Silent);
+        }
+    }
+
     QStringList otherStr = getLsFiles(dir, QStringList() << "--others" << "--" << source.toLocalFile(), KDevelop::OutputJob::Silent);
     if(otherStr.isEmpty()) {
         DVcsJob* job = new DVcsJob(dir, this, KDevelop::OutputJob::Verbose);
@@ -1399,3 +1429,12 @@ void GitPlugin::delayedBranchChanged()
 {
     emit repositoryBranchChanged(m_branchesChange.takeFirst());
 }
+
+CheckInRepositoryJob* GitPlugin::isInRepository(KTextEditor::Document* document)
+{
+    CheckInRepositoryJob* job = new GitPluginCheckInRepositoryJob(document, repositoryRoot(document->url()).path());
+    job->start();
+    return job;
+}
+
+
