@@ -23,6 +23,7 @@
 #define KDEVPLATFORM_ASSISTANTPOPUP_H
 
 #include <QDeclarativeView>
+#include <QShortcut>
 #include <interfaces/iassistant.h>
 #include <ksharedptr.h>
 
@@ -35,22 +36,25 @@ class Cursor;
 class AssistantPopupConfig : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(QColor foreground READ foreground CONSTANT)
-    Q_PROPERTY(QColor background READ background CONSTANT)
-    Q_PROPERTY(QColor highlight READ highlight CONSTANT)
-    Q_PROPERTY(QString title READ title CONSTANT)
-    Q_PROPERTY(QList<QObject*> model READ model CONSTANT)
+    Q_PROPERTY(QColor foreground READ foreground NOTIFY colorsChanged)
+    Q_PROPERTY(QColor background READ background NOTIFY colorsChanged)
+    Q_PROPERTY(QColor highlight READ highlight NOTIFY colorsChanged)
+
+    Q_PROPERTY(QString title READ title NOTIFY titleChanged)
+    Q_PROPERTY(QList<QObject*> model READ model NOTIFY modelChanged)
     Q_PROPERTY(bool active READ isActive WRITE setActive NOTIFY activeChanged)
 
 public:
     explicit AssistantPopupConfig(QObject *parent = 0);
+
     QColor foreground() const { return m_foreground; }
     QColor background() const { return m_background; }
     QColor highlight() const { return m_highlight; }
+
     QString title() const { return m_title; }
+    void setTitle(const QString& title);
     QList<QObject*> model() const { return m_model; }
-    void setTitle(const QString& text) { m_title = text; }
-    void setModel(const QList<QObject*>& model) { m_model = model; }
+    void setModel(const QList<QObject*>& model);
 
     void setColorsFromView(QObject *view);
 
@@ -58,17 +62,20 @@ public:
     void setActive(bool active);
 
 signals:
+    void colorsChanged();
+
+    void titleChanged(const QString& title);
+    void modelChanged(const QList<QObject*>& model);
     void activeChanged(bool active);
 
 private:
     QColor m_foreground;
     QColor m_background;
     QColor m_highlight;
+
     QString m_title;
     QList<QObject*> m_model;
     bool m_active;
-
-    friend class AssistantPopup;
 };
 
 Q_DECLARE_METATYPE(AssistantPopupConfig*)
@@ -79,46 +86,46 @@ class AssistantPopup : public QDeclarativeView
 
 public:
     typedef KSharedPtr<AssistantPopup> Ptr;
+
     /**
-     * @p widget The widget below which the assistant should be shown.
      * The current main window will be used as parent widget for the popup.
      * This is to make use of the maximal space available and prevent any lines
      * in e.g. the editor to be hidden by the popup.
      */
-    AssistantPopup(KTextEditor::View *widget, const KDevelop::IAssistant::Ptr &assistant);
-    /**
-     * @brief Like creating a new assistant, but faster.
-     */
-    void reset(KTextEditor::View *widget, const KDevelop::IAssistant::Ptr &assistant);
-    KDevelop::IAssistant::Ptr assistant() const;
+    AssistantPopup();
 
-public slots:
-    void executeHideAction();
-    void notifyReopened(bool reopened=true);
+    /**
+     * Reset this popup for view @p view and show assistant @p assistant
+     *
+     * @p view The widget below which the assistant should be shown.
+     */
+    void reset(KTextEditor::View *view, const KDevelop::IAssistant::Ptr &assistant);
+
+    KDevelop::IAssistant::Ptr assistant() const;
 
 private slots:
     void updatePosition(KTextEditor::View* view, const KTextEditor::Cursor& newPos);
+    void updateState();
+
+    void executeHideAction();
+    void hideAssistant();
 
 protected:
     virtual bool eventFilter(QObject* object, QEvent* event);
-    virtual void keyPressEvent(QKeyEvent* event);
-    virtual void keyReleaseEvent(QKeyEvent* event);
     virtual bool viewportEvent(QEvent *event);
 
 private:
-    void updateActions();
-
-    /**
-     * @brief Get the geometry of the inner part (with the text) of the KTextEditor::View being used.
-     */
-    QRect textWidgetGeometry(KTextEditor::View *view) const;
+    void setView(KTextEditor::View* view);
+    void setAssistant(const KDevelop::IAssistant::Ptr& assistant);
+    void setActive( bool active );
 
     KDevelop::IAssistant::Ptr m_assistant;
-    QList<KDevelop::IAssistantAction::Ptr> m_assistantActions;
     QPointer<KTextEditor::View> m_view;
-    QScopedPointer<AssistantPopupConfig> m_config;
+    AssistantPopupConfig* m_config;
     bool m_shownAtBottom;
     bool m_reopening;
+    QTimer* m_updateTimer;
+    QList<QShortcut*> m_shortcuts;
 };
 
 #endif // KDEVPLATFORM_ASSISTANTPOPUP_H
