@@ -1,34 +1,41 @@
-#!/bin/bash
+#!/bin/sh
 
 # This file is part of KDevelop
 # Copyright 2011 David Nolden <david.nolden.kdevelop@art-master.de>
-# 
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
-# 
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Library General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU Library General Public License
 # along with this library; see the file COPYING.LIB.  If not, write to
 # the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-source ~/.bashrc
+# NOTE: While this script is more or less portable, it uses exclamation
+# mark in function names, which goes beyond the SUS/POSIX specs. This is
+# known to break with dash (Debian shell) at least.
 
-if ! [ "$APPLICATION_HOST" ]; then
+if [ -n "$KDEV_REAL_ENV" ]; then
+    # Since this runs as a replacement for the init-file, we need to chain in the 'real' one
+    . "$KDEV_REAL_ENV"
+fi
+
+if [ -z "$APPLICATION_HOST" ]; then
     export APPLICATION_HOST=$(hostname)
 fi
 
-if ! [ "$KDEV_SHELL_ENVIRONMENT_ID" ]; then
+if [ -z "$KDEV_SHELL_ENVIRONMENT_ID" ]; then
     export KDEV_SHELL_ENVIRONMENT_ID="default"
 fi
 
-if ! [ "$KDEV_DBUS_ID" ]; then
+if [ -z "$KDEV_DBUS_ID" ]; then
     echo "The required environment variable KDEV_DBUS_ID is not set. This variable defines the dbus id of the application instance instance which is supposed to be attached."
     exit 5
 fi
@@ -44,9 +51,9 @@ if ! [ -e "$KDEV_BASEDIR/kdev_dbus_socket_transformer" ]; then
 fi
 
 # Takes a list of tools, and prints a warning of one of them is not available in the path
-function checkToolsInPath {
-    for TOOL in $@; do
-        if ! [ "$(which $TOOL 2> /dev/null)" ]; then
+checkToolsInPath() {
+    for TOOL in "$@"; do
+        if [ -z "$(command -v $TOOL 2> /dev/null)" ]; then
             echo "The utility $TOOL is not in your path, the shell integration will not work properly."
         fi
     done
@@ -55,36 +62,36 @@ function checkToolsInPath {
 # Check if all required tools are there (on the host machine)
 checkToolsInPath sed qdbus ls cut dirname mktemp basename readlink hostname
 
-if ! [ "$KDEV_SSH_FORWARD_CHAIN" ]; then
+if [ -n "$KDEV_SSH_FORWARD_CHAIN" ]; then
     # Check for additional utilities that are required on the client machine
     checkToolsInPath kioclient
 fi
 
 # Queries the session name from the running application instance
-function getSessionName {
-    echo "$(qdbus $KDEV_DBUS_ID /kdevelop/SessionController org.kdevelop.kdevelop.KDevelop.SessionController.sessionName)"
+getSessionName() {
+    qdbus "$KDEV_DBUS_ID" /kdevelop/SessionController org.kdevelop.kdevelop.KDevelop.SessionController.sessionName
 }
 
-function getSessionDir {
-    echo "$(qdbus $KDEV_DBUS_ID /kdevelop/SessionController org.kdevelop.kdevelop.KDevelop.SessionController.sessionDir)"
+getSessionDir() {
+    qdbus "$KDEV_DBUS_ID" /kdevelop/SessionController org.kdevelop.kdevelop.KDevelop.SessionController.sessionDir
 }
 
-function getCurrentShellEnvPath {
-    local ENV_ID=$KDEV_SHELL_ENVIRONMENT_ID
-    if [ "$1" ]; then
-        ENV_ID=$1
+getCurrentShellEnvPath() {
+    local ENV_ID="$KDEV_SHELL_ENVIRONMENT_ID"
+    if [ -n "$1" ]; then
+        ENV_ID="$1"
     fi
-    
+
     echo "$(getSessionDir)/${ENV_ID}.sh"
 }
 
-function help! {
+help! () {
     echo "You are controlling the $APPLICATION session '$(getSessionName)'"
     echo ""
-    if [ "$1" == "" ]; then
+    if [ "X$1" = X ]; then
     echo "Standard commands:"
     echo "raise!                                 - Raise the window."
-    echo "sync!                                  - Synchronize the working directory with the currently open document."
+    echo "sync!                                  - Synchronize the working directory with the currently open document. See \"help! sync\""
     echo "open!   [file] ...                     - Open the file(s) within the attached application. See \"help! open\""
     echo "eopen!  [file] ...                     - Open the file(s) within an external application using kde-open."
     echo "create!  [file] [[text]]               - Create and open a new file."
@@ -93,13 +100,15 @@ function help! {
     echo "ssh!  [ssh arguments]                  - Connect to a remote host via ssh, keeping the control-connection alive. See \"help! remote\""
     echo ""
     echo "help!                                  - Show help."
-    echo "help! remote                           - Show help about remote shell-integration through ssh."
-    echo "help! env                              - Show help about the environment."
+    echo "help! open                             - Show extended help about file opening commands."
+    echo "help! sync                             - Show extended help about path synchronization commands."
+    echo "help! remote                           - Show extended help about remote shell-integration through ssh."
+    echo "help! env                              - Show extended help about the environment."
     echo ""
-    echo "Commands can be abbreviated by the first character(s), eg. r! instead of raise!, and se! instead of search!."
+    echo "Most commands can be abbreviated by the first character(s), eg. r! instead of raise!, and se! instead of search!."
     fi
 
-    if [ "$1" == "open" ]; then
+    if [ X"$1" = "Xopen" ]; then
     echo "Extended opening:"
     echo "The open! command can also be used to open files in specific tool-view configurations, by adding split-separators:"
     echo "- Files around the / separator will be arranged horizontally by split-view."
@@ -114,9 +123,25 @@ function help! {
     echo "                                      file1 is opened in the left view, file2 in the right upper view, and file3 in the right lower view."
     echo "open! / file1                       - The active view is split horizontally."
     echo "                                    - The active document is kept in the left split-view, and file1 is opened in the right split-view."
+    echo ""
+    echo "Short forms: o! = open!, eo! = eopen!, c! = create!"
     fi
-    
-    if [ "$1" == "remote" ]; then
+
+    if [ "X$1" = "Xsync" ]; then
+    echo "Extended syncing:"
+    echo "sync!    [[project-name]]           - If no project-name is given, then the sync! command synchronizes to the currently active document."
+    echo "                                      If no document is active, then it synchronizes to the currently selected item in the project tree-view."
+    echo "                                      If a case-insensitive project name prefix is given, then it synchronizes to the base folder of the matching project."
+    echo "syncsel!                            - Synchronizes to the currently selected item in the project tree-view, independently of the active document."
+    echo "project! [[project-name]]           - Map from a path within the build directory to the corresponding path in the source directory."
+    echo "                                      If we're already in the source directory, map to the root of the surrounding project."
+    echo "bdir!    [[project-name]]           - Map from a path within the source directory to the corresponding path in the build directory."
+    echo "                                      If we're already in the build directory, map to the root of the build directory."
+    echo ""
+    echo "Short forms: s! = sync!, ss! = syncsel!, p! = project!, b! = bdir!"
+    fi
+
+    if [ "X$1" = "Xremote" ]; then
     echo "Extended remote commands:"
     echo "ssh!  [ssh arguments]                  - Connect to a remote host via ssh, keeping the control-connection alive."
     echo "                                       - The whole dbus environment is forwarded, KDevelop needs to be installed on both sides."
@@ -127,114 +152,148 @@ function help! {
     echo "                                       - The files will be COPIED to the client machine if required."
     echo "copytohost! [client path] [host path]  - Copy a file/directory through the fish protocol from the client machine th the host machine."
     echo "copytoclient! [host path] [client path]- Copy a file/directory through the fish protocol from the host machine to the client machine."
+    echo ""
+    echo "Short forms: e! = exec!, ce! = cexec!, cth! = copytohost!, ctc! = copytoclient!"
     fi
-    
-    if [ "$1" == "env" ]; then
+
+    if [ "X$1" = "Xenv" ]; then
+      echo "Environment management:"
+      echo "The environment can be used to store session-specific macros and generally manipulate the shell environment"
+      echo "for embedded shell sessions. The environment is sourced into the shell when the shell is initialized, and"
+      echo "whenever setenv! is called."
+      echo ""
       echo "env!                                 - List all available shell environment-ids for this session."
       echo "setenv! [id]                         - Set the shell environmnet-id for this session to the given id, or update the current one."
-      echo "showenv! [id]                        - Show the current shell environment or the one with the optionally given id."
       echo "editenv! [id]                        - Edit the current shell environment or the one with the optionally given id."
+      echo "showenv! [id]                        - Show the current shell environment or the one with the optionally given id."
+      echo ""
+      echo "Short forms: sev! = setenv!, ee! = editenv!, shenv! = showenv!"
     fi
-    
     echo ""
 }
 
 # Short versions of the commands:
 
-function r! {
-    raise! $@
+r! () {
+    "raise!" "$@"
 }
 
-function s! {
-    sync! $@
+s! () {
+    "sync!" "$@"
 }
 
-function o! {
-    open! $@
+ss! () {
+    "syncsel!"
 }
 
-function eo! {
-    eopen! $@
+syncsel! () {
+    "sync!" '[selection]'
 }
 
-function e! {
-    exec! $@
+p! () {
+    if [ $# -gt 0 ]; then
+        "s!" "$@"
+    fi
+    "project!"
 }
 
-function ce! {
-    cexec! $@
+b! () {
+    if [ $# -gt 0 ]; then
+        "s!" "$@"
+    fi
+    "bdir!"
 }
 
-function c! {
-    create! $@
+o! () {
+    "open!" "$@"
 }
 
-function se! {
-    search! $@
+eo! () {
+    "eopen!" "$@"
 }
 
-function ds! {
-    dsearch! $@
+e! () {
+    "exec!" "$@"
 }
 
-function h! {
-    help! $@
+ce! () {
+    "cexec!" "$@"
 }
 
-function cth! {
-    copytohost! $@  
+c! () {
+    "create!" "$@"
 }
 
-function ctc! {
-    copytoclient! $@
+se! () {
+    "search!" "$@"
 }
 
-function sev! {
-    setenv! $@
+ds! () {
+    "dsearch!" "$@"
 }
 
-function ee! {
-    editenv! $@
+h! () {
+    "help!" "$@"
+}
+
+cth! () {
+    "copytohost!" "$@"
+}
+
+ctc! () {
+    "copytoclient!" "$@"
+}
+
+sev! () {
+    "setenv!" "$@"
+}
+
+ee! () {
+    "editenv!" "$@"
+}
+
+shev! () {
+    "showenv!" "$@"
 }
 
 # Internals:
 
 # Opens a document in internally in the application
-function openDocument {
+openDocument () {
     RESULT=$(qdbus $KDEV_DBUS_ID /org/kdevelop/DocumentController org.kdevelop.DocumentController.openDocumentSimple $1)
-    if ! [ "$RESULT" == "true" ]; then
+    if ! [ "X$RESULT" == "Xtrue" ]; then
         echo "Failed to open $1"
     fi
 }
 
 # Opens a document in internally in the application
-function openDocuments {
+openDocuments () {
     RESULT=$(qdbus $KDEV_DBUS_ID /org/kdevelop/DocumentController org.kdevelop.DocumentController.openDocumentsSimple "(" $1 ")")
-    if ! [ "$RESULT" == "true" ]; then
+    if ! [ "X$RESULT" == "Xtrue" ]; then
         echo "Failed to open $1"
     fi
 }
 
 # Executes a command on the client machine using the custom-script integration.
 # First argument: The full command. Second argument: The working directory.
-function executeInApp {
+executeInApp () {
     local CMD="$1"
     local WD=$2
-    if ! [ "$WD" ]; then
+    if [ -z "$WD" ]; then
         WD=$(pwd)
     fi
     RESULT=$(qdbus $KDEV_DBUS_ID /org/kdevelop/ExternalScriptPlugin org.kdevelop.ExternalScriptPlugin.executeCommand "$CMD" "$WD")
-    if ! [ "$RESULT" == "true" ]; then
+    if [ "X$RESULT" == "Xtrue" ]; then
         echo "Execution failed"
     fi
 }
 
 # First argument: The full command. Second argument: The working directory.
 # Executes the command silently and synchronously, and returns the output
-function executeInAppSync {
+executeInAppSync () {
     local CMD=$1
     local WD=$2
-    if ! [ "$WD" ]; then
+    if [ -z "$WD" ]; then
         WD=$(pwd)
     fi
     RESULT=$(qdbus $KDEV_DBUS_ID /org/kdevelop/ExternalScriptPlugin org.kdevelop.ExternalScriptPlugin.executeCommandSync "$CMD" "$WD")
@@ -243,30 +302,49 @@ function executeInAppSync {
 
 # Getter functions:
 
-function getActiveDocument {
-    qdbus $KDEV_DBUS_ID /org/kdevelop/DocumentController org.kdevelop.DocumentController.activeDocumentPath
+getActiveDocument () {
+    qdbus "$KDEV_DBUS_ID" /org/kdevelop/DocumentController org.kdevelop.DocumentController.activeDocumentPath "$@"
 }
 
-function getOpenDocuments {
-    qdbus $KDEV_DBUS_ID /org/kdevelop/DocumentController org.kdevelop.DocumentController.activeDocumentPaths
+getOpenDocuments () {
+    qdbus "$KDEV_DBUS_ID" /org/kdevelop/DocumentController org.kdevelop.DocumentController.activeDocumentPaths
 }
 
-function raise! {
-    qdbus $KDEV_DBUS_ID /kdevelop/MainWindow org.kdevelop.MainWindow.ensureVisible
+raise! () {
+    qdbus "$KDEV_DBUS_ID" /kdevelop/MainWindow org.kdevelop.MainWindow.ensureVisible
+}
+
+bdir! () {
+    TARG=$(qdbus "$KDEV_DBUS_ID" /org/kdevelop/ProjectController org.kdevelop.ProjectController.mapSourceBuild "$(pwd)" false)
+    if [ -n "$TARG" ]; then
+        cd -- "$TARG"
+    else
+        echo "Got no path"
+    fi
+}
+
+project! () {
+    TARG=$(qdbus "$KDEV_DBUS_ID" /org/kdevelop/ProjectController org.kdevelop.ProjectController.mapSourceBuild "$(pwd)" true)
+    if [ -n "$TARG" ]; then
+        cd -- "$TARG"
+    else
+        echo "Got no path"
+    fi
 }
 
 
 # Main functions:
 
-function raise! {
-    qdbus $KDEV_DBUS_ID /kdevelop/MainWindow org.kdevelop.MainWindow.ensureVisible
+raise!() {
+    qdbus "$KDEV_DBUS_ID" /kdevelop/MainWindow org.kdevelop.MainWindow.ensureVisible
 }
 
-function sync! {
-    local P=$(getActiveDocument)
-    if [ "$P" ]; then
-        
-        if [[ "$P" == fish://* ]]; then
+sync!() {
+    local P=$(getActiveDocument $@)
+    if [ -n "$P" ]; then
+
+        case $P in
+        fish://*)
             # This regular expression filters the user@host:port out of fish:///user@host:port/path/...
             LOGIN=$(echo $P | sed "s/fish\:\/\/*\([^\/]*\)\(\/.*\)/\1/")
             P_ON_HOST=$(echo $P | sed "s/fish\:\/\/*\([^\/]*\)\(\/.*\)/\2/")
@@ -285,7 +363,7 @@ function sync! {
                         PORT=$(echo $LOGIN | cut --delimiter=':' -f 2)
                         CMD="$CMD -p $PORT"
                     fi
-                    
+
                     CMD="$CMD $HOST"
                     # Execute the ssh command
                     echo "Executing $CMD"
@@ -297,17 +375,21 @@ function sync! {
                     return
                 fi
             fi
-            
-        elif [ "$KDEV_SSH_FORWARD_CHAIN" ]; then
-            # This session is being forwarded to another machine, but the current document is not
-            # However, we won't complain, because it's possible that the machines share the same file-system
-            if [ $(isEqualFileOnHostAndClient $P) != "yes" ]; then
-                echo "Cannot synchronize the working directory, because the file systems do not match"
-                return
+            ;;
+        *)
+            if [ "$KDEV_SSH_FORWARD_CHAIN" ]; then
+                # This session is being forwarded to another machine, but the current document is not
+                # However, we won't complain, because it's possible that the machines share the same file-system
+                if [ $(isEqualFileOnHostAndClient $P) != "yes" ]; then
+                    echo "Cannot synchronize the working directory, because the file systems do not match"
+                    return
+                fi
             fi
-        fi
-        
-        cd $(dirname $P)
+            ;;
+        esac
+
+        [ -d "$P" ] || P=$(dirname "$P")
+        cd -- "$P"
     else
         echo "Got no path"
     fi
@@ -315,14 +397,14 @@ function sync! {
 
 # Take a path, and returns "yes" if the equal file is available on the host and the client
 # The check is performed by comparing inode-numbers
-function isEqualFileOnHostAndClient {
-    function trimWhiteSpace() {
-        echo $1
+isEqualFileOnHostAndClient() {
+    trimWhiteSpace() {
+        printf "%s" "$1"
     }
 
     FILE=$1
-    INODE_HOST=$(trimWhiteSpace $(ls --color=never -i $FILE | cut -d' ' -f1))
-    INODE_CLIENT=$(trimWhiteSpace $(executeInAppSync "ls --color=never -i $FILE | cut -d' ' -f1" "$(dirname $FILE)"))
+    INODE_HOST=$(trimWhiteSpace $(/bin/ls -i "$FILE" | cut -d' ' -f1) )
+    INODE_CLIENT=$(trimWhiteSpace $(executeInAppSync "/bin/ls -i $FILE | cut -d' ' -f1" "$(dirname "$FILE")"))
     if [ "$INODE_HOST" == "$INODE_CLIENT" ]; then
         echo "yes"
     else
@@ -331,25 +413,27 @@ function isEqualFileOnHostAndClient {
 }
 
 # Takes a relative file, returns an absolute file/url that should be valid on the client.
-function mapFileToClient {
+mapFileToClient() {
     local RELATIVE_FILE=$1
-    FILE=$(readlink -f $RELATIVE_FILE)
+    FILE=$(readlink -f "$RELATIVE_FILE")
     if ! [ -e "$FILE" ]; then
         # Try opening the file anyway, it might be an url or something else we don't understand here
         FILE=$RELATIVE_FILE
     else
         # We are referencing an absolute file, available on the file-system.
-        
+
         if [ "$KDEV_SSH_FORWARD_CHAIN" ]; then
             # If we are forwarding, map it to the client somehow.
             if [ "$(isEqualFileOnHostAndClient "$FILE")" != "yes" ]; then
                     # We can eventually map the file using the fish protocol
                     FISH_HOST=$KDEV_SSH_FORWARD_CHAIN
-                    if [[ "$FISH_HOST" == *\,* ]]; then
+                    case $FISH_HOST in
+                    *\,*)
                         # Extracts everything before the first comma
                         FISH_HOST=$(echo $FISH_HOST | sed 's/\([^,]*\),\(.*\)/\1/')
                         echo "ssh chain is too long: $KDEV_SSH_FORWARD_CHAIN mapping anyway using $FISH_HOST" 1>&2
-                    fi
+                        ;;
+                    esac
                     # Theoretically, we can only map through fish if the forward-chains contains no comma, which means that
                     # we forward only once. Try anyway, there might be the same filesystem on the whole forward-chain.
                     FILE="fish://$FISH_HOST$FILE"
@@ -359,7 +443,8 @@ function mapFileToClient {
     echo $FILE
 }
 
-function open! {
+open!() {
+    # would break on files with whitespace in names
     FILES=$@
     NEWFILES=""
     for RELATIVE_FILE in $FILES; do
@@ -374,7 +459,8 @@ function open! {
     openDocuments "$NEWFILES"
 }
 
-function eopen! {
+eopen!() {
+    # would break on files with whitespace in names
     FILES=$@
     for RELATIVE_FILE in $FILES; do
         FILE=$(mapFileToClient $RELATIVE_FILE)
@@ -382,7 +468,8 @@ function eopen! {
     done
 }
 
-function exec! {
+exec!() {
+    # would break on files with whitespace in names
     FILES=$@
     ARGS=""
     for RELATIVE_FILE in $FILES; do
@@ -398,15 +485,16 @@ function exec! {
     executeInApp "$ARGS"
 }
 
-function copytohost! {
+copytohost!() {
     executeInApp "kioclient copy $1 $(mapFileToClient $2)"
 }
 
-function copytoclient! {
+copytoclient!() {
     executeInApp "kioclient copy $(mapFileToClient $1) $2"
 }
 
-function cexec! {
+cexec!() {
+    # would break on files with whitespace in names
     FILES=$@
     ARGS=""
     PREFIX=""
@@ -417,16 +505,16 @@ function cexec! {
             ARGS=$RELATIVE_FILE
         else
             FILE=$(mapFileToClient $RELATIVE_FILE)
-            
+
             if [[ "$FILE" == fish://* ]]; then
                 # Add a prefix to copy the file into a temporary file
                 # Keep the baseline as suffix, so that applications can easily recognize the mimetype
                 PREFIX+="FILE$TMP=\$(mktemp).$(basename $FILE); kioclient copy $FILE \$FILE$TMP;"
-                # Use the temporary variable instead of the name 
+                # Use the temporary variable instead of the name
                 FILE="\$FILE$TMP"
                 TMP=$(($TMP+1))
             fi
-            
+
             ARGS=$ARGS" "$FILE
         fi
     done
@@ -434,9 +522,9 @@ function cexec! {
     executeInApp "$PREFIX $ARGS"
 }
 
-function create! {
-    FILE=$(readlink -f $1)
-    if ! [ "$FILE" ]; then
+create!() {
+    FILE=$(readlink -f "$1")
+    if [ -z "$FILE" ]; then
         echo "Error: Bad arguments."
         return 1
     fi
@@ -444,14 +532,14 @@ function create! {
         echo "The file $FILE already exists"
         return 2
     fi
-    echo $2 > $FILE
-    
+    echo "$2" > $FILE
+
     openDocument $(mapFileToClient $FILE)
 }
 
-function search! {
+search!() {
     PATTERN=$1
-    
+
 #     if ! [ "$PATTERN" ]; then
 #         echo "Error: No pattern given."
 #         return 1
@@ -459,28 +547,28 @@ function search! {
 
     LOCATION=$2
 
-    if ! [ "$LOCATION" ]; then
+    if [ -z "$LOCATION" ]; then
         LOCATION="."
     fi
-    
-    LOCATION=$(mapFileToClient $LOCATION)
-    
-    for LOC in $*; do
-        if [ "$LOC" == "$1" ]; then
+
+    LOCATION=$(mapFileToClient "$LOCATION")
+
+    for LOC in "$@"; do
+        if [ "X$LOC" == "X$1" ]; then
             continue;
         fi
-        if [ "$LOC" == "$2" ]; then
+        if [ "X$LOC" == "X$2" ]; then
             continue;
         fi
-        LOCATION="$LOCATION;$(mapFileToClient $LOC)"
+        LOCATION="$LOCATION;$(mapFileToClient "$LOC")"
     done
-    
-    qdbus $KDEV_DBUS_ID /org/kdevelop/GrepViewPlugin org.kdevelop.kdevelop.GrepViewPlugin.startSearch "$PATTERN" "$LOCATION" true
+
+    qdbus "$KDEV_DBUS_ID" /org/kdevelop/GrepViewPlugin org.kdevelop.kdevelop.GrepViewPlugin.startSearch "$PATTERN" "$LOCATION" true
 }
 
-function dsearch! {
+dsearch!() {
     PATTERN=$1
-    
+
     if ! [ "$PATTERN" ]; then
         echo "Error: No pattern given."
         return 1
@@ -491,20 +579,20 @@ function dsearch! {
     if ! [ "$LOCATION" ]; then
         LOCATION="."
     fi
-    
-    LOCATION=$(mapFileToClient $LOCATION)
-    
-    for LOC in $*; do
-        if [ "$LOC" == "$1" ]; then
+
+    LOCATION=$(mapFileToClient "$LOCATION")
+
+    for LOC in "$@"; do
+        if [ "X$LOC" == "X$1" ]; then
             continue;
         fi
-        if [ "$LOC" == "$2" ]; then
+        if [ "X$LOC" == "X$2" ]; then
             continue;
         fi
-        LOCATION="$LOCATION;$(mapFileToClient $LOC)"
+        LOCATION="$LOCATION;$(mapFileToClient "$LOC")"
     done
-    
-    qdbus $KDEV_DBUS_ID /org/kdevelop/GrepViewPlugin org.kdevelop.kdevelop.GrepViewPlugin.startSearch "$PATTERN" "$LOCATION" false
+
+    qdbus "$KDEV_DBUS_ID" /org/kdevelop/GrepViewPlugin org.kdevelop.kdevelop.GrepViewPlugin.startSearch "$PATTERN" "$LOCATION" false
 }
 
 ##### SSH DBUS FORWARDING --------------------------------------------------------------------------------------------------------------------
@@ -515,10 +603,6 @@ DBUS_SOCKET_TRANSFORMER=$KDEV_BASEDIR/kdev_dbus_socket_transformer
 # This configures the shell to kill background jobs when it is terminated
 shopt -s huponexit
 
-# TODO: This random number can lead to conflicts, but since only ssh notices these conflicts
-#       during forwarding, it is very hard to deal with them.
-export DBUS_FORWARDING_TCP_TARGET_PORT=$((5000+($RANDOM%50000)))
-
 export DBUS_ABSTRACT_SOCKET_TARGET_BASE_PATH=/tmp/dbus-forwarded-$USER-$APPLICATION_HOST
 
 export DBUS_FORWARDING_TCP_LOCAL_PORT=9000
@@ -526,12 +610,12 @@ export DBUS_FORWARDING_TCP_MAX_LOCAL_PORT=10000
 export DBUS_ABSTRACT_SOCKET_TARGET_INDEX=1
 export DBUS_ABSTRACT_SOCKET_MAX_TARGET_INDEX=1000
 
-function getPortFromSSHCommand {
+getPortFromSSHCommand() {
     # The port is given to ssh exclusively in the format "-p PORT"
     # This regular expression extracts the "4821" from "ssh -q bla1 -p 4821 bla2"
     local ARGS=$@
     local RET=$(echo "$@" | sed "s/.*-p \+\([0-9]*\).*/\1/")
-    if [ "$ARGS" == "$RET" ]; then
+    if [ "X$ARGS" == "X$RET" ]; then
         # There was no match
         echo ""
     else
@@ -539,12 +623,12 @@ function getPortFromSSHCommand {
     fi
 }
 
-function getLoginFromSSHCommand {
+getLoginFromSSHCommand() {
     # The login name can be given to ssh in the format "-l NAME"
     # This regular expression extracts the "NAME" from "ssh -q bla1 -l NAME bla2"
     local ARGS=$@
     local RET=$(echo "$ARGS" | sed "s/.*-l \+\([a-z,A-Z,_,0-9]*\).*/\1/")
-    if [ "$RET" == "$ARGS" ] || [ "$RET" == "" ]; then
+    if [ "X$RET" == "X$ARGS" -o -z "$RET" ]; then
         # There was no match
         echo ""
     else
@@ -552,28 +636,29 @@ function getLoginFromSSHCommand {
     fi
 }
 
-function getHostFromSSHCommand {
+getHostFromSSHCommand() {
     # This regular expression extracts the "bla2" from "echo "ssh -q bla1 -p 4821 bla2"
     # Specifically, it finds the first argument which is not preceded by a "-x" parameter kind specification.
-    
+    # XXX This will break if you'd call, say, "ssh -v host ..."
+
     local CLEANED=""
     local NEWCLEANED="$@"
 
-    while ! [ "$NEWCLEANED" == "$CLEANED" ]; do
+    while ! [ "X$NEWCLEANED" == "X$CLEANED" ]; do
         CLEANED="$NEWCLEANED"
     # This expression removes one "-x ARG" parameter
-        NEWCLEANED="$(echo $CLEANED | sed "s/\(.*\)\(-[a-z,A-Z] \+[a-z,0-9]*\)\ \(.*\)/\1\3/")"
+        NEWCLEANED="$(echo "$CLEANED" | sed "s/\(.*\)\(-[a-z,A-Z] \+[a-z,0-9]*\)\ \(.*\)/\1\3/")"
     done
 
     # After cleaning, the result should only consist of the host-name followed by an optional command.
     # Select the host-name, by extracting the forst column.
-    echo $CLEANED | cut --delimiter=" " -f 1
+    echo "$CLEANED" | cut -d" " -f 1
 }
 
-function getSSHForwardOptionsFromCommand {
-    
+getSSHForwardOptionsFromCommand() {
+
     HOST="$(getLoginFromSSHCommand "$@")$(getHostFromSSHCommand "$@")$(getPortFromSSHCommand "$@")"
-    
+
     if [ "$KDEV_SSH_FORWARD_CHAIN" ]; then
         # We are already forwarding, so we deal with a chain of multiple ssh commands.
         # We still record it, although it's not sure if we can use it somehow.
@@ -583,13 +668,13 @@ function getSSHForwardOptionsFromCommand {
     fi
 }
 
-function getDBusAbstractSocketSuffix {
+getDBusAbstractSocketSuffix() {
     # From something like DBUS_SESSION_BUS_ADDRESS=unix:abstract=/tmp/dbus-wYmSkVH7FE,guid=b214dad39e0292a4299778d64d761a5b
     # extract the /tmp/dbus-wYmSkVH7FE
     echo $DBUS_SESSION_BUS_ADDRESS | sed 's/unix\:abstract\=.*\(,guid\=.*\)/\1/'
 }
 
-function keepForwardingDBusToTCPSocket {
+keepForwardingDBusToTCPSocket() {
     while ! $KDEV_BASEDIR/kdev_dbus_socket_transformer $DBUS_FORWARDING_TCP_LOCAL_PORT --bind-only; do
         if (($DBUS_FORWARDING_TCP_LOCAL_PORT<$DBUS_FORWARDING_TCP_MAX_LOCAL_PORT)); then
             export DBUS_FORWARDING_TCP_LOCAL_PORT=$(($DBUS_FORWARDING_TCP_LOCAL_PORT+1))
@@ -599,12 +684,12 @@ function keepForwardingDBusToTCPSocket {
             return 1;
         fi
     done
-        
+
     $KDEV_BASEDIR/kdev_dbus_socket_transformer $DBUS_FORWARDING_TCP_LOCAL_PORT&
     return 0;
 }
 
-function keepForwardingDBusFromTCPSocket {
+keepForwardingDBusFromTCPSocket() {
 
     while ! $KDEV_BASEDIR/kdev_dbus_socket_transformer $FORWARD_DBUS_FROM_PORT ${DBUS_ABSTRACT_SOCKET_TARGET_BASE_PATH}-${DBUS_ABSTRACT_SOCKET_TARGET_INDEX} --bind-only; do
         if ((${DBUS_ABSTRACT_SOCKET_TARGET_INDEX}<${DBUS_ABSTRACT_SOCKET_MAX_TARGET_INDEX})); then
@@ -614,16 +699,18 @@ function keepForwardingDBusFromTCPSocket {
             return 1;
         fi
     done
-    
+
     local PATH=${DBUS_ABSTRACT_SOCKET_TARGET_BASE_PATH}-${DBUS_ABSTRACT_SOCKET_TARGET_INDEX}
     export DBUS_SESSION_BUS_ADDRESS=unix:abstract=$PATH${DBUS_SOCKET_SUFFIX}
     $KDEV_BASEDIR/kdev_dbus_socket_transformer $FORWARD_DBUS_FROM_PORT $PATH&
 }
 
-function ssh! {
+ssh! () {
+    # XXX This entire function is broken by design
     keepForwardingDBusToTCPSocket # Start the dbus forwarding subprocess
-    
-    ssh $@ -t -R localhost:$DBUS_FORWARDING_TCP_TARGET_PORT:localhost:$DBUS_FORWARDING_TCP_LOCAL_PORT \
+    DBUS_FORWARDING_TCP_TARGET_PORT=$((5000+($RANDOM%50000)))
+
+    ssh "$@" -t -R localhost:$DBUS_FORWARDING_TCP_TARGET_PORT:localhost:$DBUS_FORWARDING_TCP_LOCAL_PORT \
          " APPLICATION=$APPLICATION \
            KDEV_BASEDIR=$KDEV_BASEDIR \
            KDEV_DBUS_ID=$KDEV_DBUS_ID \
@@ -655,17 +742,17 @@ function ssh! {
 }
 
 # A version of ssh! that preserves the current working directory
-function ssw! {
+ssw! () {
     KDEV_WORKING_DIR=$(pwd)
-    ssh! $@
+    ssh! "$@"
 }
 
-function env! {
+env! () {
     FILES="$(executeInAppSync "ls $(getSessionDir)/*.sh" "")"
     for FILE in $FILES; do
-        FILE=$(basename $FILE)
+        FILE=$(basename "$FILE")
         ID=${FILE%.sh} # This ugly construct strips away the .sh suffix
-        if [ "$ID" == "$KDEV_SHELL_ENVIRONMENT_ID" ]; then
+        if [ "X$ID" == "X$KDEV_SHELL_ENVIRONMENT_ID" ]; then
             echo "$ID   [current]"
         else
             echo "$ID"
@@ -673,7 +760,7 @@ function env! {
     done
 }
 
-function editenv! {
+editenv! () {
     local ENV_ID=$KDEV_SHELL_ENVIRONMENT_ID
     if [ "$1" ]; then
         ENV_ID=$1
@@ -684,13 +771,14 @@ function editenv! {
     openDocument "$(getCurrentShellEnvPath $ENV_ID)"
 }
 
-function setenv! {
+setenv! () {
     if [ "$1" ]; then
         KDEV_SHELL_ENVIRONMENT_ID=$1
     fi
-        
+
     # Execute the contents of the shell-environment
-    local TEMP=$(mktemp)
+    # note: keep compatible with FreeBSD: https://bugs.kde.org/show_bug.cgi?id=311186
+    local TEMP=$(mktemp /tmp/$USER-XXXXXXXX)
     RESULT=$(executeInAppSync "cat \"$(getCurrentShellEnvPath)\"" "")
     echo "$RESULT" > $TEMP
     if ! [ "$RESULT" ]; then
@@ -701,9 +789,9 @@ function setenv! {
     rm $TEMP
 }
 
-function showenv! {
+showenv! () {
     local ENV_ID=$KDEV_SHELL_ENVIRONMENT_ID
-    if [ "$1" ]; then
+    if [ -n "$1" ]; then
         ENV_ID=$1
     fi
 
@@ -713,13 +801,13 @@ function showenv! {
     echo $(executeInAppSync "cat \"$(getCurrentShellEnvPath $ENV_ID)\"" "")
 }
 
-if [ "$FORWARD_DBUS_FROM_PORT" ]; then
+if [ -n "$FORWARD_DBUS_FROM_PORT" ]; then
     # Start the target-side dbus forwarding, transforming from the ssh pipe to the abstract unix domain socket
     export DBUS_SESSION_BUS_ADDRESS=unix:abstract=${DBUS_ABSTRACT_SOCKET_TARGET_BASE_PATH}-${DBUS_ABSTRACT_SOCKET_TARGET_INDEX}${DBUS_SOCKET_SUFFIX}
     keepForwardingDBusFromTCPSocket
 fi
 
-setenv!
+"setenv!"
 
 ##### INITIALIZATION --------------------------------------------------------------------------------------------------------------------
 
@@ -728,6 +816,6 @@ PS1="!$PS1"
 
 echo "You are controlling the $APPLICATION session '$(getSessionName)'. Type help! for more information."
 
-if [ "$KDEV_WORKING_DIR" ]; then
-    cd $KDEV_WORKING_DIR
+if [ -n "$KDEV_WORKING_DIR" ]; then
+    cd -- "$KDEV_WORKING_DIR"
 fi
